@@ -10,12 +10,12 @@
 #' thereby possible to construct data frames for dot plots of selected
 #' coefficients, as well as density plots of the indirect effect.
 #'
-#' @name fortify.testMediation
+#' @name fortify.test_mediation
 #'
-#' @param model  an object inheriting from class \code{"\link{testMediation}"}
+#' @param model  an object inheriting from class \code{"\link{test_mediation}"}
 #' containing results from (robust) mediation analysis.
-#' @param data  for the \code{"bootTestMediation"} method, this is currently
-#' ignored.  For the \code{"sobelTestMediation"} method, this is an optional
+#' @param data  for the \code{"boot_test_mediation"} method, this is currently
+#' ignored.  For the \code{"sobel_test_mediation"} method, this is an optional
 #' numeric vector containing the \eqn{x}-values at which to evaluate the
 #' assumed normal density from Sobel's test (only used in case of a density
 #' plot).  The default is to take 100 equally spaced points between the
@@ -23,9 +23,11 @@
 #' according to Sobel's formula.
 #' @param method  a character string specifying for which plot to construct the
 #' data frame.  Possible values are \code{"dot"} for a dot plot of selected
-#' coefficients, or \code{"density"} for a density plot of the indirect effect.
+#' coefficients, or \code{"density"} for a density plot of the indirect
+#' effect(s).
 #' @param parm  a character string specifying the coefficients to be included
-#' in a dot plot.  The default is to include the direct and the indirect effect.
+#' in a dot plot.  The default is to include the direct and the indirect
+#' effect(s).
 #' @param level  numeric;  the confidence level of the confidence intervals
 #' from Sobel's test to be included in a dot plot.  The default is to include
 #' 95\% confidence intervals.
@@ -36,25 +38,53 @@
 #'
 #' @author Andreas Alfons
 #'
-#' @seealso \code{\link{testMediation}}, \code{\link{plotMediation}}
+#' @seealso \code{\link{test_mediation}}, \code{\link{plot_mediation}}
+#'
+#' @examples
+#' data("BSG2014")
+#'
+#' # run fast and robust bootstrap test
+#' test <- test_mediation(BSG2014,
+#'                        x = "ValueDiversity",
+#'                        y = "TeamCommitment",
+#'                        m = "TaskConflict")
+#'
+#' # data for dot plot
+#' dot <- fortify(test, method = "dot")
+#' plot_mediation(dot)
+#'
+#' # data for density plot
+#' density <- fortify(test, method = "density")
+#' plot_mediation(density)
 #'
 #' @keywords utilities
 
 NULL
 
 
-#' @rdname fortify.testMediation
-#' @method fortify bootTestMediation
+# For multiple densities (indirect effects from different mediators), add a
+# column that identifies the effect and add a formula for facetting
+
+# For dotplots with multiple mediators, simply add more dots/errorbars in the
+# same plot
+
+#' @rdname fortify.test_mediation
+#' @method fortify boot_test_mediation
 #' @import ggplot2
 #' @export
 
-fortify.bootTestMediation <- function(model, data,
-                                      method = c("dot", "density"),
-                                      parm = c("c", "ab"), ...) {
+fortify.boot_test_mediation <- function(model, data,
+                                        method = c("dot", "density"),
+                                        parm = NULL, ...) {
   # initialization
   method <- match.arg(method)
+  p_m <- length(model$fit$m)
   # construct data fram with relevant information
   if(method == "dot") {
+    if(is.null(parm)) {
+      if(p_m == 1L) parm <- c("c", "ab")
+      else parm <- c("c", paste("ab", names(model$ab), sep = "_"))
+    }
     # extract point estimates
     coef <- coefficients(model, parm=parm)
     # extract confidence intervals
@@ -70,18 +100,37 @@ fortify.bootTestMediation <- function(model, data,
                                         ymin="Lower", ymax="Upper")
     attr(data, "geom") <- geom_pointrange
   } else {
-    # construct data frame containing bootstrap density
-    pdf <- density(model$reps$t[, 1])
-    data <- data.frame(ab=pdf$x, Density=pdf$y)
     # extract point estimate and confidence interval
     ab <- model$ab
     ci <- model$ci
+    # construct data frame containing bootstrap density
+    if(p_m == 1L) {
+      pdf <- density(model$reps$t[, 1L])
+      data <- data.frame(ab = pdf$x, Density = pdf$y)
+    } else {
+      pdf <- lapply(seq_len(1L + p_m), function(j) density(model$reps$t[, j]))
+      data <- mapply(function(density, effect) {
+        data.frame(ab = density$x, Density = density$y, Effect = effect)
+      }, density = pdf, effect = names(ab), SIMPLIFY = FALSE, USE.NAMES = FALSE)
+      data <- do.call(rbind, data)
+    }
     # add additional information as attributes
-    attr(data, "mapping") <- aes_string(x="ab", y="Density")
-    attr(data, "geom") <- function(..., stat) geom_density(..., stat="identity")
+    attr(data, "mapping") <- aes_string(x = "ab", y = "Density")
+    attr(data, "geom") <- function(..., stat) {
+      geom_density(..., stat = "identity")
+    }
     attr(data, "main") <- "Bootstrap distribution"
-    attr(data, "ci") <- data.frame(ab, Density=NA_real_,
-                                   Lower=ci[1], Upper=ci[2])
+    if(p_m == 1) {
+      attr(data, "ci") <- data.frame(ab = ab, Density = NA_real_,
+                                     Lower = ci[1L], Upper = ci[2L])
+    } else {
+      attr(data, "ci") <- data.frame(ab = unname(ab),
+                                     Density = NA_real_,
+                                     Lower = unname(ci[, 1L]),
+                                     Upper = unname(ci[, 2L]),
+                                     Effect = names(ab))
+      attr(data, "facets") <- ~ Effect  # split plot into different panels
+    }
   }
   # return data frame
   attr(data, "method") <- method
@@ -89,21 +138,22 @@ fortify.bootTestMediation <- function(model, data,
 }
 
 
-#' @rdname fortify.testMediation
-#' @method fortify sobelTestMediation
+#' @rdname fortify.test_mediation
+#' @method fortify sobel_test_mediation
 #' @import ggplot2
 #' @export
 
-fortify.sobelTestMediation <- function(model, data,
-                                       method = c("dot", "density"),
-                                       parm = c("c", "ab"),
-                                       level = 0.95, ...) {
+fortify.sobel_test_mediation <- function(model, data,
+                                         method = c("dot", "density"),
+                                         parm = NULL,
+                                         level = 0.95, ...) {
   # initialization
   method <- match.arg(method)
   level <- rep(as.numeric(level), length.out=1)
   if(is.na(level) || level < 0 || level > 1) level <- formals()$level
   # construct data fram with relevant information
   if(method == "dot") {
+    if(is.null(parm)) parm <- c("c", "ab")
     # extract point estimates
     coef <- coefficients(model, parm=parm)
     # extract confidence intervals
@@ -123,7 +173,7 @@ fortify.sobelTestMediation <- function(model, data,
     ab <- model$ab
     se <- model$se
     # compute confidence interval
-    ci <- confintZ(ab, se, level=level, alternative=model$alternative)
+    ci <- confint_z(ab, se, level=level, alternative=model$alternative)
     # x- and y-values for the density
     if(missing(data)) x <- seq(ab - 3 * se, ab + 3 * se, length.out=100)
     else x <- as.numeric(data)
@@ -131,7 +181,9 @@ fortify.sobelTestMediation <- function(model, data,
     data <- data.frame(ab=x, Density=y)
     # add additional information as attributes
     attr(data, "mapping") <- aes_string(x="ab", y="Density")
-    attr(data, "geom") <- function(..., stat) geom_density(..., stat="identity")
+    attr(data, "geom") <- function(..., stat) {
+      geom_density(..., stat = "identity")
+    }
     attr(data, "main") <- "Assumed normal distribution"
     attr(data, "ci") <- data.frame(ab, Density=dnorm(ab, mean=ab, sd=se),
                                    Lower=ci[1], Upper=ci[2])
@@ -142,18 +194,23 @@ fortify.sobelTestMediation <- function(model, data,
 }
 
 
-#' @rdname fortify.testMediation
+#' @rdname fortify.test_mediation
 #' @method fortify list
 #' @import ggplot2
 #' @export
 
 fortify.list <- function(model, data, ...) {
   ## initializations
-  isBoot <- sapply(model, inherits, "bootTestMediation")
-  isSobel <- sapply(model, inherits, "sobelTestMediation")
-  model <- model[isBoot | isSobel]
+  is_boot <- sapply(model, inherits, "boot_test_mediation")
+  is_sobel <- sapply(model, inherits, "sobel_test_mediation")
+  model <- model[is_boot | is_sobel]
   if(length(model) == 0) {
-    stop('no objects inheriting from class "testMediation"')
+    stop('no objects inheriting from class "test_mediation"')
+  }
+  # check if all objects use the same number of mediators
+  p_m <- sapply(model, function(object) length(object$fit$m))
+  if(length(unique(p_m)) > 1) {
+    stop("all objects must use the same number of hypothesized mediators")
   }
   # check names of list elements
   methods <- names(model)
@@ -164,17 +221,17 @@ fortify.list <- function(model, data, ...) {
   }
   ## fortify each list element
   if(missing(data)) data <- lapply(model, fortify, ...)
-  else data <- lapply(model, fortify, data=data, ...)
+  else data <- lapply(model, fortify, data = data, ...) # is this actually used?
   ## extract additional information
-  info <- lapply(data, function(x) attributes(x)[-(1:3)])
+  info <- lapply(data, function(x) attributes(x)[-(1L:3L)])
   if(info[[1]]$method == "density") ci <- lapply(info, "[[", "ci")
   info <- info[[1]]
   ## combine information into one data frame
   data <- mapply(function(x, method) {
-    cbind(Method=rep.int(method, nrow(x)), x, stringsAsFactors=FALSE)
-  }, data, methods, SIMPLIFY=FALSE, USE.NAMES=FALSE)
+    cbind(Method = rep.int(method, nrow(x)), x, stringsAsFactors = FALSE)
+  }, data, methods, SIMPLIFY = FALSE, USE.NAMES = FALSE)
   data <- do.call(rbind, data)
-  data$Method <- factor(data$Method, levels=methods)
+  data$Method <- factor(data$Method, levels = methods)
   ## modify additional information
   if(info$method == "dot") {
     # additional information for dot plot
@@ -188,14 +245,16 @@ fortify.list <- function(model, data, ...) {
     }
   } else {
     # additional information for density plot
-    info$mapping <- aes_string(x="ab", y="Density", color="Method")
-    if(any(isBoot) && any(isSobel)) {
-      info$geom <- function(..., stat) geom_density(..., stat="identity")
+    info$mapping <- aes_string(x = "ab", y = "Density", color = "Method")
+    if(any(is_boot) && any(is_sobel)) {
+      info$geom <- function(..., stat) {
+        geom_density(..., stat = "identity")
+      }
       info$main <- NULL
     }
     # combine confidence intervals for the methods
     ci <- do.call(rbind, ci)
-    ci <- cbind(Method=factor(methods, levels=methods), ci)
+    ci <- cbind(Method = factor(methods, levels = methods), ci)
     rownames(ci) <- NULL
     info$ci <- ci
   }
